/*
 * Project name:
     DHT22 click (Demonstration of mikroBUS DHT22 click board)
 * Copyright:
     (c) Mikroelektronika, 2015.
 * Revision History:
       - initial release (FJ);
 * Description:
     This code demonstrates how to use DHT22 click with EasyPIC v7 board.
     DHT22 sensor measures temperature and relative humidity.
 * Test configuration:
     MCU:             PIC18F45K22
                      http://ww1.microchip.com/downloads/en/DeviceDoc/41412F.pdf
     Dev.Board:       EasyPIC7
                      http://www.mikroe.com/easypic/
     Oscillator:      HS-PLL 32.0000 MHz, 8.0000 MHz Crystal
     SW:              mikroC PRO for PIC
                      http://www.mikroe.com/mikroc/pic/
 * NOTES:
     - Place DHT22 click board at the mikroBUS socket 1 on the EasyPIC v7 board.
     - Put LCD display in its socket, turn on backlight using the SW4.6
 */

// LCD module connections
sbit LCD_RS at LATB4_bit;
sbit LCD_EN at LATB5_bit;
sbit LCD_D4 at LATB0_bit;
sbit LCD_D5 at LATB1_bit;
sbit LCD_D6 at LATB2_bit;
sbit LCD_D7 at LATB3_bit;

sbit LCD_RS_Direction at TRISB4_bit;
sbit LCD_EN_Direction at TRISB5_bit;
sbit LCD_D4_Direction at TRISB0_bit;
sbit LCD_D5_Direction at TRISB1_bit;
sbit LCD_D6_Direction at TRISB2_bit;
sbit LCD_D7_Direction at TRISB3_bit;
// End LCD module connections

// AM2302 module connections
sbit AM2302_Bus_In at RE0_bit;
sbit AM2302_Bus_Out at RE0_bit;
sbit AM2302_Bus_Direction at TRISE0_bit;
// END AM2302 module connections

char AM2302_Data[5] = {0, 0, 0, 0, 0};
unsigned humidity = 0, temperature = 0;

//Get Sensor values
char AM2302_Read(unsigned *humidity, unsigned *temperature) {
  char i = 0, j = 1;
  char timeout = 0;
  char sensor_byte;
  AM2302_Bus_Out = 0;
  delay_ms(100);
  
  AM2302_Bus_Direction = 1;                         //Set AM2302_Bus as input
  AM2302_Bus_Direction = 0;                         //Set AM2302_Bus as output
  AM2302_Bus_Out = 1;                               //Set AM2302_Bus HIGH for 100 milliseconds
  Delay_ms(100);                                    //Delay 100ms
  AM2302_Bus_Out = 0;                               //Host the start signal down time min: 0.8ms, typ: 1ms, max: 20ms
  Delay_ms(2);                                      //Delay 2ms
  AM2302_Bus_Out = 1;                               //Set AM2302_Bus HIGH
  AM2302_Bus_Direction = 1;                         //Set AM2302_Bus as input

  // Bus master has released time min: 20us, typ: 30us, max: 200us
  timeout = 200;
  while (AM2302_Bus_In == 1) {
    Delay_us(1);
    if (!timeout--) {
      return 1;
    } //ERROR: Sensor not responding
  }

  // AM2302 response signal min: 75us, typ: 80us, max: 85us
  while (!AM2302_Bus_In) { //response to low time
    Delay_us(85);
  }

  while (AM2302_Bus_In) {  //response to high time
    Delay_us(55);
  }
  
  /*
   * time in us:            min  typ  max
   * signal 0 high time:    22   26   30 (bit=0)
   * signal 1 high time:    68   70   75 (bit=1)
   * signal 0,1 down time:  48   50   55
   */
   
  i = 0; //get 5 byte
  for (i = 0; i < 5; i++) {
    j = 1;
    for (j = 1; j <= 8; j++) { //get 8 bits from sensor
      while (!AM2302_Bus_In) { //signal "0", "1" low time
        Delay_us(1);
      }
      Delay_us(30);
      sensor_byte <<= 1;       //add new lower byte
      if (AM2302_Bus_In) {     //if sda high after 30us => bit=1 else bit=0
        sensor_byte |= 1;
        delay_us(45);
        while (AM2302_Bus_In) {
          Delay_us(1);
        }
      }
    }
    AM2302_Data[i] = sensor_byte;
  }
  
  *humidity = (AM2302_Data[0] << 8) + AM2302_Data[1];
  *temperature = (AM2302_Data[2] << 8) + AM2302_Data[3];

  return 0;
}

// Process and display current value to LCD
void processValue(unsigned humidity, unsigned temperature) {
  char txt[15];
  float temp;
  
  LCD_Cmd(_LCD_CLEAR);
  Lcd_Cmd(_LCD_CURSOR_OFF);

  temp = humidity / 10.0;
  Lcd_Out(1,1,"Humid.:");
  sprintf(txt, "%2.1f", temp);    // Format Relative_humidity_val and store it to txt
  Lcd_Out(1,9,txt);
  Lcd_Chr(1,14,0x25);

  temp = temperature / 10.0;
  Lcd_Out(2,1,"Temp. :");
  sprintf(txt, "%2.1f", temp);    // Format Temperature and store it to txt
  Lcd_Out(2,9,txt);
  Lcd_Chr(2,13,0xB2);
  Lcd_Chr(2,14,0x43);
}

void Display_Init(){
  LCD_Init();
  LCD_Cmd(_LCD_CLEAR);
  Lcd_Cmd(_LCD_CURSOR_OFF);
  
  // Display string on the LCD
  Lcd_Out(1,3,"DHT22  click");
  Lcd_Out(2,3,"Demo Example");
  delay_ms(2000);
}

void main() {
  char k = 0, t;
  ANSELE = 0;
  
  Display_Init();

  while (1) {
    if (AM2302_Read(&humidity, &temperature) == 0)                 // Display AM2302_Read sensor values via LCD
      processValue(humidity, temperature);                         // Display AM2302_Read sensor values via LCD
    else {
      LCD_Cmd(_LCD_CLEAR);
      Lcd_Cmd(_LCD_CURSOR_OFF);
      Lcd_Out(1,3,"Sensor not");
      Lcd_Out(2,3,"responding");
    }
    Delay_ms(2000); //Delay 2000ms
  }
}